@@ -2,9 +2,9 @@
 
 #include "Python.h"
 
-static char cprt[] = 
+static const char cprt[] =
 "\
-Copyright (c) 2001-2015 Python Software Foundation.\n\
+Copyright (c) 2001-2014 Python Software Foundation.\n\
 All Rights Reserved.\n\
 \n\
 Copyright (c) 2000 BeOpen.com.\n\
@@ -19,5 +19,5 @@ All Rights Reserved.";
 const char *
 Py_GetCopyright(void)
 {
-	return cprt;
+    return cprt;
 }
