@@ -460,7 +460,7 @@ PyImport_Cleanup(void)
     while (PyDict_Next(modules, &pos, &key, &value)) {
         if (PyModule_Check(value)) {
             if (Py_VerboseFlag && PyUnicode_Check(key))
-                PySys_FormatStderr("# cleanup[2] removing %U\n", key, value);
+                PySys_FormatStderr("# cleanup[2] removing %U\n", key);
             STORE_MODULE_WEAKREF(key, value);
             PyDict_SetItem(modules, key, Py_None);
         }
@@ -837,12 +837,10 @@ error:
     return m;
 }
 
-PyObject*
-PyImport_ExecCodeModuleObject(PyObject *name, PyObject *co, PyObject *pathname,
-                              PyObject *cpathname)
+static PyObject *
+module_dict_for_exec(PyObject *name)
 {
-    PyObject *modules = PyImport_GetModuleDict();
-    PyObject *m, *d, *v;
+    PyObject *m, *d = NULL;
 
     m = PyImport_AddModuleObject(name);
     if (m == NULL)
@@ -852,31 +850,26 @@ PyImport_ExecCodeModuleObject(PyObject *name, PyObject *co, PyObject *pathname,
     d = PyModule_GetDict(m);
     if (PyDict_GetItemString(d, "__builtins__") == NULL) {
         if (PyDict_SetItemString(d, "__builtins__",
-                                 PyEval_GetBuiltins()) != 0)
-            goto error;
+                                 PyEval_GetBuiltins()) != 0) {
+            remove_module(name);
+            return NULL;
+        }
     }
-    if (pathname != NULL) {
-        v = pathname;
-    }
-    else {
-        v = ((PyCodeObject *)co)->co_filename;
-    }
-    Py_INCREF(v);
-    if (PyDict_SetItemString(d, "__file__", v) != 0)
-        PyErr_Clear(); /* Not important enough to report */
-    Py_DECREF(v);
 
-    /* Remember the pyc path name as the __cached__ attribute. */
-    if (cpathname != NULL)
-        v = cpathname;
-    else
-        v = Py_None;
-    if (PyDict_SetItemString(d, "__cached__", v) != 0)
-        PyErr_Clear(); /* Not important enough to report */
+    return d;  /* Return a borrowed reference. */
+}
 
-    v = PyEval_EvalCode(co, d, d);
-    if (v == NULL)
-        goto error;
+static PyObject *
+exec_code_in_module(PyObject *name, PyObject *module_dict, PyObject *code_object)
+{
+    PyObject *modules = PyImport_GetModuleDict();
+    PyObject *v, *m;
+
+    v = PyEval_EvalCode(code_object, module_dict, module_dict);
+    if (v == NULL) {
+        remove_module(name);
+        return NULL;
+    }
     Py_DECREF(v);
 
     if ((m = PyDict_GetItem(modules, name)) == NULL) {
@@ -889,10 +882,32 @@ PyImport_ExecCodeModuleObject(PyObject *name, PyObject *co, PyObject *pathname,
     Py_INCREF(m);
 
     return m;
+}
 
-  error:
-    remove_module(name);
-    return NULL;
+PyObject*
+PyImport_ExecCodeModuleObject(PyObject *name, PyObject *co, PyObject *pathname,
+                              PyObject *cpathname)
+{
+    PyObject *d, *res;
+    PyInterpreterState *interp = PyThreadState_GET()->interp;
+    _Py_IDENTIFIER(_fix_up_module);
+
+    d = module_dict_for_exec(name);
+    if (d == NULL) {
+        return NULL;
+    }
+
+    if (pathname == NULL) {
+        pathname = ((PyCodeObject *)co)->co_filename;
+    }
+    res = _PyObject_CallMethodIdObjArgs(interp->importlib,
+                                        &PyId__fix_up_module,
+                                        d, name, pathname, cpathname, NULL);
+    if (res != NULL) {
+        Py_DECREF(res);
+        res = exec_code_in_module(name, d, co);
+    }
+    return res;
 }
 
 
@@ -1206,7 +1221,7 @@ int
 PyImport_ImportFrozenModuleObject(PyObject *name)
 {
     const struct _frozen *p;
-    PyObject *co, *m, *path;
+    PyObject *co, *m, *d;
     int ispackage;
     int size;
 
@@ -1235,7 +1250,7 @@ PyImport_ImportFrozenModuleObject(PyObject *name)
     }
     if (ispackage) {
         /* Set __path__ to the empty list */
-        PyObject *d, *l;
+        PyObject *l;
         int err;
         m = PyImport_AddModuleObject(name);
         if (m == NULL)
@@ -1250,11 +1265,11 @@ PyImport_ImportFrozenModuleObject(PyObject *name)
         if (err != 0)
             goto err_return;
     }
-    path = PyUnicode_FromString("<frozen>");
-    if (path == NULL)
+    d = module_dict_for_exec(name);
+    if (d == NULL) {
         goto err_return;
-    m = PyImport_ExecCodeModuleObject(name, co, path, NULL);
-    Py_DECREF(path);
+    }
+    m = exec_code_in_module(name, d, co);
     if (m == NULL)
         goto err_return;
     Py_DECREF(co);
